<?php

class Zend_Mail_Encryption_SMIME {
    /**
     * Path to a directory writeable by PHP user for storing temporary files
     * @var string
     */
    protected $_tmpDir = './tmp';
    
    /**
     * Filename of either certificate (encryption) or private key and certificate (signature)
     * @var string
     */
    protected $_privKeyCert = '';
    
    /**
     * Password for certificate
     * @var string
     */
    protected $_privKeyPassword = '';
    
    /**
     * Constructor. Accepts certificate and/or (depending whether you're 
     * signing or encrypting your email) in PEM format and the password for the key
     * Third parameter is a directory writeable by PHP user. This is needed because
     * PHP openssl_pkcs7_* functions works only with files.
     *
     * @param string $privKeyCert Filename of either certificate (encryption) or private key and certificate (signature)
     * @param string $privKeyPassword Password for the certificate
     * @param string $tmpDir Path to a directory writeable by PHP user for storing temporary files
     * @return void
     */
    public function __construct($privKeyCert, $privKeyPassword = '', $tmpDir = 'tmp')
    {
        if(!file_exists($privKeyCert)) {
            throw new Zend_Mail_Exception('privKeyCert must be a file in PEM format');
        }
        $this->_privKeyCert = 'file://'.realpath($privKeyCert);
        $this->_privKeyPassword = $privKeyPassword;
        
        $this->setTmpDir($tmpDir);
    }
    
    /*
     * Set/change temporary directory
     * 
     * @param string $tmpDir path to a directory writeable by PHP user for storing temporary files
     * @return Zend_Mail_Encryption_SMIME
     * @throws Zend_Mail_Exception
     */
    public function setTmpDir($tmpDir)
    {
        if(($dir = realpath($tmpDir)) !== false) {
            $this->_tmpDir = $dir;
        } else {
            throw new Zend_Mail_Exception('Temporary directory must be writeable by current user');
        }
        
        return $this;
    }

    /*
     * Get temporary directory
     * 
     * @return string $tmpDir path to a directory writeable by PHP user for storing temporary files
     */
    public function getTmpDir()
    {
        return $this->_tmpDir;
    }
    
    /*
     * Gets email message as parameter and signs it using PKCS7 algoritm
     * @param string $message Email message (all headers + body)
     * @return array ('headers' => split headers of signed message, 'body' => body of signed message)
     */
    public function signMessage($message)
    {
        // important: this "\n" is a fix for MS Outlook. See http://ua.php.net/manual/en/function.openssl-pkcs7-sign.php#36038
        file_put_contents($this->getTmpDir() . DIRECTORY_SEPARATOR . 'message',$message.Zend_Mime::LINEEND);
        openssl_pkcs7_sign($this->getTmpDir() . DIRECTORY_SEPARATOR . 'message',$this->getTmpDir() . DIRECTORY_SEPARATOR . 'signed_message',$this->_privKeyCert,array($this->_privKeyCert,$this->_privKeyPassword),array(), PKCS7_DETACHED);
        
        $data = file_get_contents($this->getTmpDir() . DIRECTORY_SEPARATOR . 'signed_message');
        unlink($this->getTmpDir() . DIRECTORY_SEPARATOR . 'message');
        unlink($this->getTmpDir() . DIRECTORY_SEPARATOR . 'signed_message');
        $parts = explode(Zend_Mime::LINEEND . Zend_Mime::LINEEND, $data,2);
        
        $headers = array();
        foreach(explode(Zend_Mime::LINEEND,$parts[0]) as $header) {
            list($key, $value) = explode(':',$header,2);
            $headers[$key] = array(0 => trim($value), 'append' => 'false');
        }
        
        return array('headers' => $headers, 'body' => $parts[1]);
    }
    
    /*
     * Gets email message as parameter and encrypts it using PKCS7 algoritm
     * @param string $message Email message (all headers + body)
     * @return array ('headers' => split headers of encrypted message, 'body' => body of encrypted message)
     */
    public function encryptMessage($message)
    {
        // important: this "\n" is a fix for MS Outlook. See http://ua.php.net/manual/en/function.openssl-pkcs7-sign.php#36038
        file_put_contents($this->getTmpDir() . DIRECTORY_SEPARATOR . 'message',$message.Zend_Mime::LINEEND);
        
        openssl_pkcs7_encrypt($this->getTmpDir() . DIRECTORY_SEPARATOR . 'message',$this->getTmpDir() . DIRECTORY_SEPARATOR . 'signed_message',$this->_privKeyCert,array());
        
        $data = file_get_contents($this->getTmpDir() . DIRECTORY_SEPARATOR . 'signed_message');
        unlink($this->getTmpDir() . DIRECTORY_SEPARATOR . 'message');
        unlink($this->getTmpDir() . DIRECTORY_SEPARATOR . 'signed_message');
        $parts = explode(Zend_Mime::LINEEND . Zend_Mime::LINEEND, $data,2);
        
        $headers = array();
        foreach(explode(Zend_Mime::LINEEND,$parts[0]) as $header) {
            list($key, $value) = explode(':',$header,2);
            $headers[$key] = array(0 => trim($value), 'append' => 'false');
        }
        
        return array('headers' => $headers, 'body' => $parts[1]);
    }
}